# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import logging
from pecan import abort
from pecan import rest
import pecan
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from examples.webhooks import tasks


LOG = logging.getLogger(__name__)


class Controller(rest.RestController):
    """API root controller"""

    @wsme_pecan.wsexpose(wtypes.text)
    def get_all(self):
        LOG.debug("Fetch items.")

        values = {
            'tasks': [
                'put_service_on_hold',
                'backup_user_data',
                'backup_service_data',
                'execute_backup'
            ]
        }

        if not values:
            abort(404)
        else:
            return values

    @wsme_pecan.wsexpose(wtypes.text, wtypes.text)
    def get(self, name):
        print("Task '%s' is running." % name)

        value = "Task %s accepted" % name
        tasks.start_task(**self.get_mistral_headers())
        return value

    def get_mistral_headers(self):
        headers = pecan.request.headers
        try:
            needed_headers = {
                'workbook_name': headers['Mistral-Workbook-Name'],
                'execution_id': headers['Mistral-Execution-Id'],
                'task_id': headers['Mistral-Task-Id']
            }
            return needed_headers
        except KeyError:
            raise RuntimeError("Could not find http headers for "
                               "defining mistral task")



# Copyright 2013 Mirantis, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import flask


app = flask.Flask(__name__)
host = "0.0.0.0"
port = 5000


@app.route('/summ', methods=["POST"])
def summ():
    summ = 0
    args = flask.request.data

    try:
        args_list = flask.json.loads(args).get('arguments')
    except:
        return (403, "Please, specify list of arguments in the request body"
                " in form '{\"arguments\": [arg1, arg2, .., argN]}'")

    for a in args_list:
        try:
            summ += int(a)
        except:
            return 403, "Sorry, need to use only integer arguments!"

    return flask.jsonify({'result': summ})


if __name__ == "__main__":
    app.run(host=host, port=port)



# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from time import sleep
import threading

from examples.webhooks.api import client


CLIENT = client.CLIENT


def start_task(**kwargs):
    thread = threading.Thread(target=finish_task, kwargs=kwargs)
    thread.start()


def finish_task(task_id, execution_id, workbook_name):
    # simulate working
    sleep(8)

    task = CLIENT.tasks.update(workbook_name, execution_id,
                               task_id, "SUCCESS")
    print("Task '%s' - SUCCESS" % task.name)



# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from oslo.config import cfg

from examples.webhooks import version


api_opts = [
    cfg.StrOpt('host', default='0.0.0.0', help='Simple-app API server host'),
    cfg.IntOpt('port', default=8988, help='Simple-app API server port')
]

CONF = cfg.CONF
CONF.register_opts(api_opts, group='api')


def parse_args(args=None, usage=None, default_config_files=None):
    CONF(args=args,
         project='mistral-demo',
         version=version,
         usage=usage,
         default_config_files=default_config_files)



# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import pkg_resources as pkg

from mistralclient.api import client

from examples.webhooks import version

MISTRAL_URL = "http://localhost:8989/v1"
CLIENT = client.Client(mistral_url=MISTRAL_URL)


WB_NAME = "myWorkbook"
TASK = "execute_backup"


def upload_workbook():
    try:
        CLIENT.workbooks.get(WB_NAME)
    except:
        CLIENT.workbooks.create(WB_NAME,
                                description="My test workbook",
                                tags=["test"])
    print("Uploading workbook definition...\n")

    definition = get_workbook_definition()
    CLIENT.workbooks.upload_definition(WB_NAME, definition)

    print definition
    print("\nUploaded.")


def get_workbook_definition():
    return open(pkg.resource_filename(version.version_info.package,
                                      "demo.yaml")).read()


def start_execution():
    import threading
    t = threading.Thread(target=CLIENT.executions.create,
                         kwargs={'workbook_name': WB_NAME,
                                 'task': TASK})
    t.start()
    return "accepted"



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""Script to start Demo API service."""

import eventlet

import logging
import os
from requests import exceptions
import sys
import threading
import asyncore
from time import sleep
from wsgiref import simple_server
from SocketServer import ThreadingMixIn
from smtpd import DebuggingServer

from oslo.config import cfg

from examples.webhooks import config
from examples.webhooks.api import app
from examples.webhooks.api import client


eventlet.monkey_patch(
    os=True, select=True, socket=True, thread=True, time=True)

logging.basicConfig(level=logging.WARN)
LOG = logging.getLogger(__name__)


def upload_wb_and_start():
    sleep(5)

    try:
        client.upload_workbook()
    except exceptions.ConnectionError:
        LOG.error("Error. Mistral service probably is not working now")
        sys.exit(1)

    print("Start execution for: %s" % client.TASK)

    client.start_execution()


class ThreadingWSGIServer(ThreadingMixIn, simple_server.WSGIServer):
    pass


def _run_smtpd():
    #TODO: get address/port from demo.yaml smtp_server to match.
    DebuggingServer(('localhost', 10025), None)
    asyncore.loop()


def main():
    try:
        config.parse_args()

        host = cfg.CONF.api.host
        port = cfg.CONF.api.port

        server = simple_server.make_server(
            host, port, app.setup_app(), ThreadingWSGIServer)

        LOG.info("Demo app API is serving on http://%s:%s (PID=%s)" %
                 (host, port, os.getpid()))
        server.serve_forever()
    except RuntimeError, e:
        sys.stderr.write("ERROR: %s\n" % e)
        sys.exit(1)
    except KeyboardInterrupt:
        sys.exit(0)


if __name__ == '__main__':
    upload_thread = threading.Thread(target=upload_wb_and_start)
    upload_thread.run()
    smtp_thread = threading.Thread(target=_run_smtpd)
    smtp_thread.daemon = True
    smtp_thread.start()
    main()



# -*- coding: utf-8 -*-
#
# Copyright 2013 - Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

import pecan


app = {
    'root': 'examples.webhooks.api.controllers.root.RootController',
    'modules': ['examples.webhooks.api'],
    'debug': True,
}


def get_pecan_config():
    # Set up the pecan configuration.
    return pecan.configuration.conf_from_dict(app)


def setup_app(config=None):
    if not config:
        config = get_pecan_config()

    app_conf = dict(config)

    return pecan.make_app(
        app_conf.pop('root'),
        logging=getattr(config, 'logging', {}),
        **app_conf
    )
